use super::bit;
use super::error::{Error, ErrorKind};
use super::result::Result;
use x86intrin::{m256i, mm256_cmpeq_epi8, mm256_movemask_epi8, mm256_setr_epi8};

#[inline]
pub fn build_structural_character_bitmap(s: &[u8], b_backslash: &mut Vec<u64>, b_quote: &mut Vec<u64>, b_colon: &mut Vec<u64>, b_left: &mut Vec<u64>, b_right: &mut Vec<u64>, m_backslash: &m256i, m_quote: &m256i, m_colon: &m256i, m_left: &m256i, m_right: &m256i) {
    let n = s.len();
    let mut i = 0;
    while i + 63 < n {
        let s1 = mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            s[i + 22] as i8,
            s[i + 23] as i8,
            s[i + 24] as i8,
            s[i + 25] as i8,
            s[i + 26] as i8,
            s[i + 27] as i8,
            s[i + 28] as i8,
            s[i + 29] as i8,
            s[i + 30] as i8,
            s[i + 31] as i8,
        );

        let s2 = mm256_setr_epi8(
            s[i + 32] as i8,
            s[i + 33] as i8,
            s[i + 34] as i8,
            s[i + 35] as i8,
            s[i + 36] as i8,
            s[i + 37] as i8,
            s[i + 38] as i8,
            s[i + 39] as i8,
            s[i + 40] as i8,
            s[i + 41] as i8,
            s[i + 42] as i8,
            s[i + 43] as i8,
            s[i + 44] as i8,
            s[i + 45] as i8,
            s[i + 46] as i8,
            s[i + 47] as i8,
            s[i + 48] as i8,
            s[i + 49] as i8,
            s[i + 50] as i8,
            s[i + 51] as i8,
            s[i + 52] as i8,
            s[i + 53] as i8,
            s[i + 54] as i8,
            s[i + 55] as i8,
            s[i + 56] as i8,
            s[i + 57] as i8,
            s[i + 58] as i8,
            s[i + 59] as i8,
            s[i + 60] as i8,
            s[i + 61] as i8,
            s[i + 62] as i8,
            s[i + 63] as i8,
        );

        b_backslash.push(mbitmap(&s1, &s2, m_backslash));
        b_quote.push(mbitmap(&s1, &s2, m_quote));
        b_colon.push(mbitmap(&s1, &s2, m_colon));
        b_left.push(mbitmap(&s1, &s2, m_left));
        b_right.push(mbitmap(&s1, &s2, m_right));

        i += 64;
    }

    if i + 32 < n {
        let s1 = mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            s[i + 22] as i8,
            s[i + 23] as i8,
            s[i + 24] as i8,
            s[i + 25] as i8,
            s[i + 26] as i8,
            s[i + 27] as i8,
            s[i + 28] as i8,
            s[i + 29] as i8,
            s[i + 30] as i8,
            s[i + 31] as i8,
        );

        let s2 = u8_to_m256i_rest(s, i + 32);

        b_backslash.push(mbitmap(&s1, &s2, m_backslash));
        b_quote.push(mbitmap(&s1, &s2, m_quote));
        b_colon.push(mbitmap(&s1, &s2, m_colon));
        b_left.push(mbitmap(&s1, &s2, m_left));
        b_right.push(mbitmap(&s1, &s2, m_right));
    } else if i + 32 == n {
        let s1 = mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            s[i + 22] as i8,
            s[i + 23] as i8,
            s[i + 24] as i8,
            s[i + 25] as i8,
            s[i + 26] as i8,
            s[i + 27] as i8,
            s[i + 28] as i8,
            s[i + 29] as i8,
            s[i + 30] as i8,
            s[i + 31] as i8,
        );

        b_backslash.push(mbitmap_partial(&s1, m_backslash));
        b_quote.push(mbitmap_partial(&s1, m_quote));
        b_colon.push(mbitmap_partial(&s1, m_colon));
        b_left.push(mbitmap_partial(&s1, m_left));
        b_right.push(mbitmap_partial(&s1, m_right));
    } else if i < n {
        let s1 = u8_to_m256i_rest(s, i);

        b_backslash.push(mbitmap_partial(&s1, m_backslash));
        b_quote.push(mbitmap_partial(&s1, m_quote));
        b_colon.push(mbitmap_partial(&s1, m_colon));
        b_left.push(mbitmap_partial(&s1, m_left));
        b_right.push(mbitmap_partial(&s1, m_right));
    }
}

#[inline]
fn mbitmap(s1: &m256i, s2: &m256i, m: &m256i) -> u64 {
    let i1 = mm256_movemask_epi8(mm256_cmpeq_epi8(*s1, *m));
    let i2 = mm256_movemask_epi8(mm256_cmpeq_epi8(*s2, *m));
    u64::from(i1 as u32) | (u64::from(i2 as u32) << 32)
}

#[inline]
fn mbitmap_partial(s: &m256i, m: &m256i) -> u64 {
    u64::from(mm256_movemask_epi8(mm256_cmpeq_epi8(*s, *m)) as u32)
}

#[inline]
fn u8_to_m256i_rest(s: &[u8], i: usize) -> m256i {
    match s.len() - i {
        31 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            s[i + 22] as i8,
            s[i + 23] as i8,
            s[i + 24] as i8,
            s[i + 25] as i8,
            s[i + 26] as i8,
            s[i + 27] as i8,
            s[i + 28] as i8,
            s[i + 29] as i8,
            s[i + 30] as i8,
            0,
        ),
        30 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            s[i + 22] as i8,
            s[i + 23] as i8,
            s[i + 24] as i8,
            s[i + 25] as i8,
            s[i + 26] as i8,
            s[i + 27] as i8,
            s[i + 28] as i8,
            s[i + 29] as i8,
            0,
            0,
        ),
        29 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            s[i + 22] as i8,
            s[i + 23] as i8,
            s[i + 24] as i8,
            s[i + 25] as i8,
            s[i + 26] as i8,
            s[i + 27] as i8,
            s[i + 28] as i8,
            0,
            0,
            0,
        ),
        28 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            s[i + 22] as i8,
            s[i + 23] as i8,
            s[i + 24] as i8,
            s[i + 25] as i8,
            s[i + 26] as i8,
            s[i + 27] as i8,
            0,
            0,
            0,
            0,
        ),
        27 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            s[i + 22] as i8,
            s[i + 23] as i8,
            s[i + 24] as i8,
            s[i + 25] as i8,
            s[i + 26] as i8,
            0,
            0,
            0,
            0,
            0,
        ),
        26 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            s[i + 22] as i8,
            s[i + 23] as i8,
            s[i + 24] as i8,
            s[i + 25] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        25 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            s[i + 22] as i8,
            s[i + 23] as i8,
            s[i + 24] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        24 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            s[i + 22] as i8,
            s[i + 23] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        23 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            s[i + 22] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        22 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            s[i + 21] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        21 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            s[i + 20] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        20 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            s[i + 19] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        19 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            s[i + 18] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        18 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            s[i + 17] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        17 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            s[i + 16] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        16 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            s[i + 15] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        15 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            s[i + 14] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        14 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            s[i + 13] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        13 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            s[i + 12] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        12 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            s[i + 11] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        11 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            s[i + 10] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        10 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            s[i + 9] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        9 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            s[i + 8] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        8 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            s[i + 7] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        7 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            s[i + 6] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        6 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            s[i + 5] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        5 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            s[i + 4] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        4 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            s[i + 3] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        3 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            s[i + 2] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        2 => mm256_setr_epi8(
            s[i] as i8,
            s[i + 1] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        1 => mm256_setr_epi8(
            s[i] as i8,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
        _ => mm256_setr_epi8(
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ),
    }
}

#[inline]
pub fn build_structural_quote_bitmap(b_backslash: &[u64], b_quote: &mut Vec<u64>) {
    let n = b_quote.len();
    if n < 1 {
        return;
    }
    let mut b_unstructural_quote = Vec::with_capacity(n);
    let mut b_backslash_quote = Vec::with_capacity(n);
    for i in 0..n - 1 {
        b_backslash_quote.push(((b_quote[i] >> 1) | b_quote[i + 1] << 63) & b_backslash[i]);
    }
    b_backslash_quote.push((b_quote[n - 1] >> 1) & b_backslash[n - 1]);
    for (i, backslash_quote) in b_backslash_quote.iter().enumerate() {
        let mut unstructural_quote = 0u64;
        let mut backslash_quote = *backslash_quote;
        while backslash_quote != 0 {
            let backslash_quote_mask = bit::s(backslash_quote);
            let backslash_quote_mask_ones_num = backslash_quote_mask.count_ones();
            let mut consecutive_backslash_num = 0;
            for j in (0..i + 1).rev() {
                let backslash_b = b_backslash[j];
                if j == i {
                    let backslash_b_mask = (backslash_b & backslash_quote_mask) << (64 - backslash_quote_mask_ones_num);
                    let leading_ones_num = (!backslash_b_mask).leading_zeros();
                    consecutive_backslash_num += leading_ones_num;
                    if leading_ones_num != backslash_quote_mask_ones_num {
                        break;
                    }
                } else {
                    let backslash_b_mask = backslash_b & 0xffff_ffff_ffff_ffffu64;
                    let leading_ones_num = (!backslash_b_mask).leading_zeros();
                    consecutive_backslash_num += leading_ones_num;
                    if leading_ones_num != 64 {
                        break;
                    }
                }
            }
            if consecutive_backslash_num & 1 == 1 {
                unstructural_quote |= bit::e(backslash_quote);
            }
            backslash_quote = bit::r(backslash_quote);
        }
        b_unstructural_quote.push(!unstructural_quote);
    }
    b_quote[0] &= b_unstructural_quote[0] << 1;
    for i in 1..n {
        b_quote[i] &= (b_unstructural_quote[i] << 1) | (b_unstructural_quote[i - 1] >> 63);
    }
}

#[inline]
pub fn build_string_mask_bitmap(b_quote: &[u64], b_string_mask: &mut Vec<u64>) {
    let mut n = 0;
    for m_quote in b_quote {
        let mut m_quote = *m_quote;
        let mut m_string = 0u64;
        while m_quote != 0 {
            let m = bit::s(m_quote);
            m_string ^= m;
            m_quote = bit::r(m_quote);
            n += 1;
        }
        if n & 1 == 0 {
            m_string = !m_string;
        }
        b_string_mask.push(m_string);
    }
}

#[inline]
pub fn build_leveled_colon_bitmap(b_colon: &[u64], b_left: &[u64], b_right: &[u64], l: usize, b: &mut Vec<Vec<u64>>) -> Result<()> {
    for _ in 0..l {
        b.push(b_colon.to_owned());
    }
    let mut s = Vec::new();
    let mut s_len = 0;
    for i in 0..b_right.len() {
        let mut m_left = b_left[i];
        let mut m_right = b_right[i];
        loop {
            let m_rightbit = bit::e(m_right);
            let mut m_leftbit = bit::e(m_left);
            while m_leftbit != 0 && (m_rightbit == 0 || m_leftbit < m_rightbit) {
                s.push((i, m_leftbit));
                s_len += 1;
                m_left = bit::r(m_left);
                m_leftbit = bit::e(m_left);
            }
            if m_rightbit != 0 {
                let (j, mlb) = match s.pop() {
                    Some(v) => v,
                    None => {
                        return Err(Error::from(ErrorKind::InvalidRecord));
                    }
                };
                s_len -= 1;
                m_leftbit = mlb;
                if s_len > 0 {
                    let upper_l = s_len - 1;
                    if upper_l < l {
                        if i == j {
                            b[upper_l][i] &= !(m_rightbit.wrapping_sub(m_leftbit));
                        } else {
                            b[upper_l][j] &= m_leftbit.wrapping_sub(1);
                            b[upper_l][i] &= !(m_rightbit.wrapping_sub(1));
                            for k in j + 1..i {
                                b[upper_l][k] = 0
                            }
                        }
                    }
                }
            }
            m_right = bit::r(m_right);
            if m_rightbit == 0 {
                break;
            }
        }
    }
    Ok(())
}

#[cfg(test)]
mod tests {
    use super::*;
    use super::super::avx;
    use super::super::utf8::QUOTE;

    #[test]
    fn test_build_structural_character_bitmap() {
        let char = QUOTE;
        let m = avx::mm256i(char as i8);
        struct TestCase {
            s: Vec<u8>,
            d: Vec<u64>,
        }
        #[allow(overflowing_literals)]
        let test_cases = vec![
            TestCase {
                s: vec![],
                d: vec![],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000010,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_01000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_10000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_11111111_11111111_11111111_11111111,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000001_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000001_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000001_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000001_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000001_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000001_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000001_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                ],
                d: vec![
                    0b10000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_11111111_11111111_11111111_11111111,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                ],
                d: vec![
                    0b11111111_11111111_11111111_11111111_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                    char,
                ],
                d: vec![
                    0b11111111_11111111_11111111_11111111_11111111_11111111_11111111_11111111,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000010_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_10000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000001_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00100000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b10000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000010_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00001000_00000000_00000000,
                ],
            },
            TestCase {
                s: vec![
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    0xff,
                    char,
                ],
                d: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_10000000_00000000_00000000_00000000,
                ],
            },
        ];
        for t in test_cases {
            let mut d = Vec::with_capacity((t.s.len() + 1) / 2);
            build_structural_character_bitmap(
                &t.s,
                &mut d,
                &mut vec![],
                &mut vec![],
                &mut vec![],
                &mut vec![],
                &m,
                &m,
                &m,
                &m,
                &m,
            );
            assert_eq!(t.d, d);
        }
    }

    #[test]
    fn test_build_structural_quote_bitmap() {
        struct TestCase {
            b_backslash: Vec<u64>,
            b_quote: Vec<u64>,
            want: Vec<u64>,
        }
        let test_cases = vec![
            TestCase {
                b_backslash: vec![],
                b_quote: vec![],
                want: vec![],
            },
            TestCase {
                b_backslash: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000010,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000010,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000100_00000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00001000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000110_00000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00001000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00001000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000111_00000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00001000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000111_10000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00001000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00001000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000111_11000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00001000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000111_11100000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00001000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00001000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b00000000_00000110_00000000_00000000_00000000_00100000_00000000_00000000,
                    0b00000000_00000000_00011110_00000000_00000000_00000000_00001110_00000000,
                ],
                b_quote: vec![
                    0b00000000_00001000_00000000_00000000_00100000_01000000_00000000_00000010,
                    0b00000000_00000000_00100000_00000000_00000000_00000000_00010000_00001000,
                ],
                want: vec![
                    0b00000000_00001000_00000000_00000000_00100000_00000000_00000000_00000010,
                    0b00000000_00000000_00100000_00000000_00000000_00000000_00000000_00001000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b10000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b11000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b11100000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b11110000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000010,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b10000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000010,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000010,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b11000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000010,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b11100000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000010,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000010,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b10001111_10000001_11111110_00000000_00000000_00000000_00000000_00000000,
                    0b10000000_00000000_00000000_00000111_00011110_00000000_00000000_00000001,
                    0b00000000_00000000_00000000_00000000_00000000_01100000_00100000_00000000,
                ],
                b_quote: vec![
                    0b00010000_00000010_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00001000_00100000_00000000_00000000_00000010,
                    0b00000000_00000000_00000000_00000000_00000000_10000000_01000000_00000001,
                ],
                want: vec![
                    0b00000000_00000010_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00100000_00000000_00000000_00000010,
                    0b00000000_00000000_00000000_00000000_00000000_10000000_00000000_00000000,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b10000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b11111111_11111111_11111111_11111111_11111111_11111111_11111111_11111111,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000010,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000010,
                ],
            },
            TestCase {
                b_backslash: vec![
                    0b11000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b11111111_11111111_11111111_11111111_11111111_11111111_11111111_11111111,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000010,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
        ];
        for t in test_cases {
            let mut b_quote = t.b_quote.clone();
            build_structural_quote_bitmap(&t.b_backslash, &mut b_quote);
            assert_eq!(t.want, b_quote);
        }
    }

    #[test]
    fn test_build_string_mask_bitmap() {
        struct TestCase {
            b_quote: Vec<u64>,
            want: Vec<u64>,
        }
        let test_cases = vec![
            TestCase {
                b_quote: vec![],
                want: vec![],
            },
            TestCase {
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                want: vec![
                    0b11111111_11111111_11111111_11111111_11111111_11111111_11111111_11111111,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000100,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000111,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_01000000_00000100,
                ],
                want: vec![
                    0b11111111_11111111_11111111_11111111_11111111_11111111_10000000_00000111,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00000000_00000000_00000000_00000010_00000000_00000000_01000000_00000100,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000011_11111111_11111111_10000000_00000111,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00000000_01000000_00000000_00000010_00000000_00000000_01000000_00000100,
                ],
                want: vec![
                    0b11111111_10000000_00000000_00000011_11111111_11111111_10000000_00000111,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                want: vec![
                    0b11111111_11111111_11111111_11111111_11111111_11111111_11111111_11111111,
                    0b11111111_11111111_11111111_11111111_11111111_11111111_11111111_11111111,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000100,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000111,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00010000_00000000_00000000_00000000_00000000_00000000_00010000_00100000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                want: vec![
                    0b00011111_11111111_11111111_11111111_11111111_11111111_11100000_00111111,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00010000_00000000_00000000_00000000_00000000_00000000_00010000_00100000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00001000,
                ],
                want: vec![
                    0b00011111_11111111_11111111_11111111_11111111_11111111_11100000_00111111,
                    0b11111111_11111111_11111111_11111111_11111111_11111111_11111111_11110000,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00010000_00000000_00000000_00000000_00000000_00000000_00010000_00100000,
                    0b00000000_00000000_00000000_00000000_10000000_00000000_00000000_00001000,
                ],
                want: vec![
                    0b00011111_11111111_11111111_11111111_11111111_11111111_11100000_00111111,
                    0b00000000_00000000_00000000_00000000_11111111_11111111_11111111_11110000,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                want: vec![
                    0b11111111_11111111_11111111_11111111_11111111_11111111_11111111_11111111,
                    0b11111111_11111111_11111111_11111111_11111111_11111111_11111111_11111111,
                    0b11111111_11111111_11111111_11111111_11111111_11111111_11111111_11111111,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000001_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000001_11111111,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000001_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00100000_00000000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000001_11111111,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b11111111_11111111_11111111_11111111_11111111_11000000_00000000_00000000,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000001_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00010000_00000000_00000000_00100000_00000000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000001_11111111,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00000000_00000000_00011111_11111111_11111111_11000000_00000000_00000000,
                ],
            },
            TestCase {
                b_quote: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000001_00000000,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b00001000_00000000_00010000_00000000_00000000_00100000_00000000_00000000,
                ],
                want: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000001_11111111,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    0b11110000_00000000_00011111_11111111_11111111_11000000_00000000_00000000,
                ],
            },
        ];
        for t in test_cases {
            let mut b_string_mask = Vec::with_capacity(t.b_quote.len());
            build_string_mask_bitmap(&t.b_quote, &mut b_string_mask);
            assert_eq!(t.want, b_string_mask);
        }
    }

    #[test]
    fn test_build_leveled_colon_bitmap() {
        struct TestCase {
            b_colon: Vec<u64>,
            b_left: Vec<u64>,
            b_right: Vec<u64>,
            l: usize,
            want: Vec<Vec<u64>>,
        }
        let test_cases = vec![
            TestCase {
                b_colon: vec![],
                b_left: vec![],
                b_right: vec![],
                l: 1,
                want: vec![vec![]],
            },
            TestCase {
                b_colon: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                b_left: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                b_right: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                l: 1,
                want: vec![
                    vec![
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                    ],
                ],
            },
            TestCase {
                b_colon: vec![
                    0b00000000_00010000_00000000_00000000_00010000_00000000_00000100_00000000,
                ],
                b_left: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
                b_right: vec![
                    0b10000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                l: 1,
                want: vec![
                    vec![
                        0b00000000_00010000_00000000_00000000_00010000_00000000_00000100_00000000,
                    ],
                ],
            },
            TestCase {
                b_colon: vec![
                    0b00000000_00010000_00000000_00000000_00010000_00000000_00000100_00000000,
                ],
                b_left: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001,
                ],
                b_right: vec![
                    0b10000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000,
                ],
                l: 2,
                want: vec![
                    vec![
                        0b00000000_00010000_00000000_00000000_00010000_00000000_00000100_00000000,
                    ],
                    vec![
                        0b00000000_00010000_00000000_00000000_00010000_00000000_00000100_00000000,
                    ],
                ],
            },
            TestCase {
                b_colon: vec![
                    0b00000000_00010000_00000000_00000000_00010000_00000000_00000100_00000000,
                ],
                b_left: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00001000_00000000_00000001,
                ],
                b_right: vec![
                    0b10000000_00000000_01000000_00000000_00000000_00000000_00000000_00000000,
                ],
                l: 1,
                want: vec![
                    vec![
                        0b00000000_00010000_00000000_00000000_00000000_00000000_00000100_00000000,
                    ],
                ],
            },
            TestCase {
                b_colon: vec![
                    0b00000000_00010000_00000000_00000000_00010000_00000000_00000100_00000000,
                ],
                b_left: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00001000_00000000_00000001,
                ],
                b_right: vec![
                    0b10000000_00000000_01000000_00000000_00000000_00000000_00000000_00000000,
                ],
                l: 2,
                want: vec![
                    vec![
                        0b00000000_00010000_00000000_00000000_00000000_00000000_00000100_00000000,
                    ],
                    vec![
                        0b00000000_00010000_00000000_00000000_00010000_00000000_00000100_00000000,
                    ],
                ],
            },
            TestCase {
                b_colon: vec![
                    0b00001000_00001000_00001000_00000000_00000000_00010000_00010000_00010000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000001_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                ],
                b_left: vec![
                    0b00000000_00000000_00000000_00000000_00000001_00000001_00000001_00000001u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                ],
                b_right: vec![
                    0b10000000_10000000_10000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b10000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                ],
                l: 3,
                want: vec![
                    vec![
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00010000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000001_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    ],
                    vec![
                        0b00001000_00000000_00000000_00000000_00000000_00000000_00010000_00010000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000001_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    ],
                    vec![
                        0b00001000_00001000_00000000_00000000_00000000_00010000_00010000_00010000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000001_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    ],
                ],
            },
            TestCase {
                b_colon: vec![
                    0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                ],
                b_left: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000001u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                ],
                b_right: vec![
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b10000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b10000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b10000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    0b10000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                ],
                l: 3,
                want: vec![
                    vec![
                        0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    ],
                    vec![
                        0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    ],
                    vec![
                        0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                        0b00000000_10000000_00000000_00000000_00000000_00000000_00000000_00000000u64,
                    ],
                ],
            },
        ];
        for t in test_cases {
            let mut b = Vec::with_capacity(t.l);
            let r = build_leveled_colon_bitmap(&t.b_colon, &t.b_left, &t.b_right, t.l, &mut b);
            assert_eq!(Ok(()), r);
            assert_eq!(t.want, b);
        }
    }
}
